﻿function GeneralController() {
    this.variationRequestHandler = function (evt) {
        var self = this;
        var dialog = new VariationDialog({ model: evt.model });
        var dialogController = {};
        _.extend(dialogController, Backbone.Events);
        dialogController.listenTo(dialog, 'variations-generated', function (variations) {
            var parent = evt.model.get('parent');
            for (var i = 0; i < variations.length; i++) {
                var title = evt.model.get('title');
                var offset = title.indexOf('(')
                if (offset != -1) {
                    title = title.substring(0, offset);
                }
                title += '(';
                for (var j = 0; j < variations[i].length; j++) {
                    title += variations[i][j].Name + " = " + variations[i][j].Value + "; ";
                }
                title += ')';
                var data = { AlgorithmParameters: variations[i] };
                if (evt.model.attributes.data.ProblemParameters) {
                    data.ProblemParameters = evt.model.attributes.data.ProblemParameters;
                }
                parent.addNode({ title: title, nodeId: evt.model.get('nodeId'), data: data });
            }
            self.model.trigger('change');
        });
        dialog.render();
    },
    this.experimentFinishedHandler = function (experimentModel, availableExperiments, selectableGroup) {
        var loadingDialog = new LoadingDialog({ el: $('#loadingDialog') });
        // start the loading dialog
        loadingDialog.setLoading(true);
        loadingDialog.text('Saving experiment ' + experimentModel.get('title') + '...');
        loadingDialog.render();
        // send experiment to server
        experimentModel.save({}, { success: function () {
            loadingDialog.setLoading(false);
            loadingDialog.text('Experiment successfully saved!');
            loadingDialog.render();
            if (availableExperiments)
                availableExperiments.fetch({ cache: false, success: function (itm) {
                    selectableGroup.render();
                }
                });
        }
        });
        //loadingDialog.close();
    }
}

function ExperimentPageController() {
    this.refreshExperiments = function () {
        var availableExperiments = new ExperimentList();
        availableExperiments.fetch({ cache: false, success: function (itm) {
            $('#experiments').empty();
            for (var i = 0; i < availableExperiments.length; i++) {
                new DraggableView({ model: availableExperiments.models[i], el: $('#experiments') }).render();
            }
        }
        });
    },
    this.refreshScenarios = function () {
        var availableScenarios = new ScenarioList();
        availableScenarios.fetch({ cache: false, success: function (itm) {
            var group = new DraggableGroup({ model: availableScenarios, el: $('#algorithms') });
            group.render();
        }
        });
    },
    this.create = function () {
        var self = this;
        var generalController = new GeneralController();
        // create all models + required view for the experiment page
        var model = new ExperimentNode({
            title: 'Experiment',
            key: 0,
            isExperiment: true
        }, { parse: true });
        generalController.model = model;
        var experimentModel = new Experiment({ experiment: model });

        var view = new ExperimentTreeView({ model: model, el: $('#container') });
        view.render();

        this.refreshScenarios();

        this.refreshExperiments();

        var wizard = new StepWizardView({ el: $('#stepWizard') });
        wizard.render();

        var detailsView = new ExperimentDetailsTreeView({ model: model, el: $('#container2') });
        var validationView = new ValidationHintsView({ model: { header: "Validation failed!", text: ""} });
        var parameterDialog = new ParameterDialog({ model: model, el: $('#parameterDialog') });

        var wizardController = {};
        _.extend(wizardController, Backbone.Events);

        wizardController.listenTo(model, 'change', function () {
            // update the details view, when the model changes
            view.render();
            detailsView.render();
        });


        wizardController.listenTo(wizard, 'experiment-finished', function () {
            generalController.experimentFinishedHandler(experimentModel, null, null);
            self.refreshExperiments();
            /*// start the loading dialog
            loadingDialog.setLoading(true);
            loadingDialog.text('Saving experiment ' + experimentModel.get('title') + '...');
            loadingDialog.render();
            // send experiment to server
            experimentModel.save({}, { success: function () {
            loadingDialog.setLoading(false);
            loadingDialog.text('Experiment successfully saved!');
            loadingDialog.render();
            self.refreshExperiments();
            }
            });*/
            //loadingDialog.close();
        });

        wizardController.listenTo(view, 'structure-changed', function (node) {
            view.render();
            detailsView.render();
        });

        wizardController.listenTo(view, 'node-added', function (node) {
            // get details about the node
            node.fetch({ success: function (itm) {
                model.trigger('change', {});
            }
            });
        });

        wizardController.listenTo(detailsView, 'node-clicked', function (node) {
            // get details about the node
            var modelNode = ExperimentNode.lookup(node.data.key);
            if (!modelNode.get('isExperiment')) {
                parameterDialog.model = modelNode;
                parameterDialog.render();
            }
        });

        wizardController.listenTo(parameterDialog, 'parameters-finished', function (node) {
            parameterDialog.close();
        });

        wizardController.listenTo(view, 'variation-request', function (evt) {
            generalController.variationRequestHandler(evt);
        });

        var experimentDetailsView = new ExperimentDetailsView({ el: $('#step3'), model: experimentModel });
    }
}



/// ==================== EDIT PAGE =========================
/// ==================== EDIT PAGE =========================
/// ==================== EDIT PAGE =========================

function ExperimentEditPageController() {
    this.refreshScenarios = function () {
        var availableScenarios = new ScenarioList();
        availableScenarios.fetch({ cache: false, success: function (itm) {
            var group = new DraggableGroup({ model: availableScenarios, el: $('#draggableAlgorithms') });
            group.render();
            /*$('#algorithms').empty();
            for (var i = 0; i < availableScenarios.length; i++) {
            new DraggableView({ model: availableScenarios.models[i], el: $('#algorithms') }).render();
            }*/
        }
        });
    },
    this.create = function () {
        var generalController = new GeneralController();

        var self = this;
        // create all models + required view for the experiment page
        var controllerModel = new ControllerModel();
        var model = new ExperimentNode({
            title: 'Experiment',
            key: 0,
            isExperiment: true
        }, { parse: true });

        generalController.model = model;
        var experimentModel = new Experiment({ experiment: model });
        var experimentDetailsView = new ExperimentDetailsView({ el: $('#step4'), model: experimentModel });
        experimentDetailsView.render();

        var availableExperiments = new ExperimentList();
        var selectableGroup = null;
        var wizardController = {};
        _.extend(wizardController, Backbone.Events);

        this.refreshScenarios();

        availableExperiments.fetch({ cache: false, success: function (itm) {
            selectableGroup = new SelectableGroup({ model: availableExperiments, el: $('#experiments') });
            selectableGroup.render();
            var draggableExperiments = new DraggableGroup({ model: availableExperiments, el: $('#draggableExperiments') });
            draggableExperiments.render();

            wizardController.listenTo(selectableGroup, 'node-selected', function (nodeId) {
                var node = availableExperiments.find(function (itm) { return itm.get('nodeId') == nodeId; });
                var newNode = new ExperimentNode(node.attributes);
                experimentModel.set({
                    experiment: newNode,
                    title: node.get('title'),
                    nodeId: node.get('nodeId'),
                    run: node.get('run'),
                    repititions: node.get('repititions'),
                    group: node.get('group')
                });
                controllerModel.set({ selectedNode: newNode });
                detailsView.model = newNode;
                detailsView.render();
                view.model = newNode;
                view.render();
                experimentDetailsView.render();
            }
            );
        }
        });

        // create wizard with 4 steps
        var wizard = new StepWizardView({ el: $('#stepWizard') });
        wizard.render();

        var view = new ExperimentTreeView({ model: model, el: $('#container') });
        view.render();
        var validationView = new ValidationHintsView({ model: { header: "Validation failed!", text: ""} });

        var detailsView = new ExperimentDetailsTreeView({ model: model, el: $('#container2') });

        var parameterDialog = new ParameterDialog({ model: model, el: $('#parameterDialog') });
        var loadingDialog = new LoadingDialog({ el: $('#loadingDialog') });

        wizardController.listenTo(wizard, 'step-validated', function (validationModel) {
            // update the details view
            if (validationModel.stepNumber == 1) {
                if (controllerModel.get('selectedNode') == null) {
                    validationView.model["text"] = 'Please select an experiment to continue!';
                    validationView.render();
                    validationModel.succeeded = false;
                }
            }
            detailsView.render();
        });

        wizardController.listenTo(model, 'change', function () {
            // update the details view, when the model changes
            view.render();
            detailsView.render();
        });

        wizardController.listenTo(view, 'structure-changed', function (node) {
            view.render();
        });

        wizardController.listenTo(view, 'node-added', function (node) {
            // get details about the node
            //if (!node.get('isExperiment'))
            node.fetch({ success: function (itm) {
                model.trigger('change', {});
            }
            });
        });

        wizardController.listenTo(wizard, 'experiment-finished', function () {
            generalController.experimentFinishedHandler(experimentModel, availableExperiments, selectableGroup);          
        });

        wizardController.listenTo(detailsView, 'node-clicked', function (node) {
            // get details about the node
            var modelNode = ExperimentNode.lookup(node.data.key);
            if (!modelNode.get('isExperiment')) {
                parameterDialog.model = modelNode;
                parameterDialog.render();
            }
        });

        wizardController.listenTo(view, 'variation-request', function (evt) {
            generalController.variationRequestHandler(evt);
        });

        wizardController.listenTo(parameterDialog, 'parameters-finished', function (node) {
            parameterDialog.close();
        });
    }
}


 
 