﻿//================== MODELS =====================
var ControllerModel = Backbone.Model.extend({
  defaults: {
    selectedNode: null  
  }
});

var Experiment = Backbone.RelationalModel.extend({   
  url: function () {
    return '/Experiment/Experiment?nodeId=' + this.get('nodeId');
  },
  defaults: {
    title: 'My Experiment',
    isExperiment: true,
    run: false,
    nodeId: null,
    repititions: 1,
    group: 'TESTAZURE'
  },
  isNew: function() { return this.get('nodeId') == null; },
  relations: [{
    key: 'experiment',
    relatedModel: 'ExperimentNode'
  }]
});

var ExperimentNode = Backbone.RelationalModel.extend({
    defaults: {
        title: 'Experiment',
        key: -1,
        data: {},
        parent: null,
        isExperiment: false,
        nodeId: null
    },
    relations: [{
        type: Backbone.HasMany,
        key: 'children',
        relatedModel: 'ExperimentNode',
        collectionType: 'ScenarioList',
        reverseRelation: {
            key: 'parent'
        }
    }],
    url: function () {
        return this.get('isExperiment') ? '/Experiment/Experiment?nodeId=' + this.get('nodeId') : '/Experiment/Scenario?nodeId=' + this.get('nodeId');
    },
    initialize: function (spec) {
        if (typeof spec.key === 'undefined' || spec.key == -1) {
            spec.key = ExperimentNode.globalKeyCount;
            this.set({ key: spec.key });
            ExperimentNode.globalKeyCount++;
        }
        else if (spec.key >= ExperimentNode.globalKeyCount)
            ExperimentNode.globalKeyCount = spec.key + 1;

        ExperimentNode.globalNodeMap[spec.key] = this;
    },
    // adds a new experimentNode to this node
    addNode: function (experimentNode) {
        if (!(experimentNode instanceof ExperimentNode)) {
            experimentNode = new ExperimentNode(experimentNode);
        }
        this.get('children').add(experimentNode);
        experimentNode.set({ parent: this });
        this.trigger('node-added', experimentNode);
        return experimentNode;
    },
    // moves an experimentNode into this node
    moveNode: function (experimentNode) {
        var parent = experimentNode.get('parent');
        if (parent != null) {
            // remove from parent
            parent.get('children').remove(experimentNode);
        }
        this.addNode(experimentNode);
    },
    // delete this node
    remove: function () {
        var parent = this.get('parent');
        if (parent != null) {
            parent.get('children').remove(this);
            this.set({ parent: null });
            // simply replace it with 'undefined'
            delete ExperimentNode.globalNodeMap[this.get('key')];
        }
        this.trigger('node-removed', this);
    },
    lookup: function (key) {
        return ExperimentNode.globalNodeMap[key];
    }
},
{
    globalKeyCount: 0,
    globalNodeMap: {},
    lookup: function (key) {
        return ExperimentNode.globalNodeMap[key];
    },
    nextGlobalKey: function () {
        var key = ExperimentNode.globalKeyCount;
        ExperimentNode.globalKeyCount++;
        return key;
    }
}
);

var ExperimentList = Backbone.Collection.extend({
  url: function () {
    return "/Experiment/ExperimentList";
  },
  model: Experiment
});

var ScenarioList = Backbone.Collection.extend({
  url: function () {
    return "/Experiment/ScenarioList";
  },
  model: ExperimentNode
});